@@ -11,8 +11,9 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from mesonbuild.templates.sampleimpl import SampleImpl
-import re
+from __future__ import annotations
+
+from mesonbuild.templates.sampleimpl import FileHeaderImpl
 
 
 lib_h_template = '''#pragma once
@@ -123,44 +124,13 @@ test('basic', exe)
 '''
 
 
-class ObjCProject(SampleImpl):
-    def __init__(self, options):
-        super().__init__()
-        self.name = options.name
-        self.version = options.version
+class ObjCProject(FileHeaderImpl):
 
-    def create_executable(self) -> None:
-        lowercase_token = re.sub(r'[^a-z0-9]', '_', self.name.lower())
-        source_name = lowercase_token + '.m'
-        open(source_name, 'w', encoding='utf-8').write(hello_objc_template.format(project_name=self.name))
-        open('meson.build', 'w', encoding='utf-8').write(
-            hello_objc_meson_template.format(project_name=self.name,
-                                             exe_name=lowercase_token,
-                                             source_name=source_name,
-                                             version=self.version))
-
-    def create_library(self) -> None:
-        lowercase_token = re.sub(r'[^a-z0-9]', '_', self.name.lower())
-        uppercase_token = lowercase_token.upper()
-        function_name = lowercase_token[0:3] + '_func'
-        test_exe_name = lowercase_token + '_test'
-        lib_h_name = lowercase_token + '.h'
-        lib_objc_name = lowercase_token + '.m'
-        test_objc_name = lowercase_token + '_test.m'
-        kwargs = {'utoken': uppercase_token,
-                  'ltoken': lowercase_token,
-                  'header_dir': lowercase_token,
-                  'function_name': function_name,
-                  'header_file': lib_h_name,
-                  'source_file': lib_objc_name,
-                  'test_source_file': test_objc_name,
-                  'test_exe_name': test_exe_name,
-                  'project_name': self.name,
-                  'lib_name': lowercase_token,
-                  'test_name': lowercase_token,
-                  'version': self.version,
-                  }
-        open(lib_h_name, 'w', encoding='utf-8').write(lib_h_template.format(**kwargs))
-        open(lib_objc_name, 'w', encoding='utf-8').write(lib_objc_template.format(**kwargs))
-        open(test_objc_name, 'w', encoding='utf-8').write(lib_objc_test_template.format(**kwargs))
-        open('meson.build', 'w', encoding='utf-8').write(lib_objc_meson_template.format(**kwargs))
+    source_ext = 'm'
+    header_ext = 'h'
+    exe_template = hello_objc_template
+    exe_meson_template = hello_objc_meson_template
+    lib_template = lib_objc_template
+    lib_header_template = lib_h_template
+    lib_test_template = lib_objc_test_template
+    lib_meson_template = lib_objc_meson_template
